import click
from .api_wrapper import request_data
from .database import new_db_connection


def update_tag(c, v, tag_list):
    click.echo("Your tag is being updated\n")
    querystring = {'limit': 5000}
    tag_data = request_data('GET', '/tags/values', params=querystring)
    try:
        for tag in tag_data['values']:
            if tag['category_name'] == str(c):
                if tag['value'] == str(v):
                    try:
                        tag_uuid = tag['uuid']
                        payload = {"action": "add", "assets": tag_list, "tags": [tag_uuid]}
                        data = request_data('POST', '/tags/assets/assignments', payload=payload)
                        click.echo("Job UUID : {}".format(data['job_uuid']))
                    except:
                        pass
    except:
        pass


def tag_checker(uuid, key, value):
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        cur = conn.cursor()
        # This needs to be changed to UUID when the api gets fixed
        cur.execute("SELECT * from tags where asset_ip='" + uuid + "' and tag_key='" + key + "' and tag_value='" + value + "';")

        rows = cur.fetchall()

        length = len(rows)
        if length != 0:
            return 'yes'
        return 'no'


def confirm_tag_exists(key, value):
    querystring = {'limit': 5000}
    tag_value_data = request_data('GET', '/tags/values', params=querystring)
    try:
        for tag in tag_value_data['values']:
            if str(tag['category_name']).lower() == str(key).lower():
                if str(tag['value']).lower() == str(value).lower():
                    return 'yes'
    except Exception as E:
        click.echo(E)


def return_tag_uuid(key, value):
    querystring = {'limit': 5000}
    tag_value_data = request_data('GET', '/tags/values', params=querystring)
    try:
        for tag in tag_value_data['values']:
            if str(tag['category_name']).lower() == str(key).lower():
                if str(tag['value']).lower() == str(value).lower():
                    return str(tag['uuid'])
                else:
                    return 'none'
    except Exception as E:
        click.echo(E)


def tag_msg():
    click.echo("Remember to run the update command if you want to use your new tag in navi")



from .api_wrapper import request_data


def get_licensed():
    data = request_data('GET', '/workbenches/asset-stats?date_range=90&filter.0.filter=is_licensed&filter.0.quality=eq&filter.0.value=true')
    number_of_assets = data['scanned']
    return number_of_assets



import click
from .database import new_db_connection
from .tag_helper import tag_checker
from .api_wrapper import request_data


@click.command(help="Adjust ACRs in Lumin by tag")
@click.option('--acr', default='', help='Set the ACR')
@click.option('--c', default='', help="Category to use")
@click.option('--v', default='', help="Value to use")
@click.option('--note', default="navi Generated", help="Enter a Note to your ACR Rule")
@click.option('-business', '-b', is_flag=True, help="Add Business Critical To ACR Change Reason(s)")
@click.option('-compliance', '-c', is_flag=True, help="Add Compliance To ACR Change Reason(s)")
@click.option('-mitigation', '-m', is_flag=True, help="Add Mitigation Controls To ACR Change Reason(s)")
@click.option('-development', '-d', is_flag=True, help="Add Development To ACR Change Reason(s)")
def lumin(acr, v, c, note, business,  compliance, mitigation, development):
    choice = []
    if c == '':
        click.echo("We require a Tag Category to update the ACR by Tag")
        exit()

    if v == '':
        click.echo("We require a Tag value to update the ACR by Tag")
        exit()

    if business:
        choice.append("Business Critical")

    if compliance:
        choice.append("In Scope For Compliance")

    if mitigation:
        choice.append("Existing Mitigation Control")

    if development:
        choice.append("Dev Only")

    if not business and not mitigation and not compliance and not development:
        choice.append("Key Drivers does not match")

    if note != 'navi Generated':
        choice.append("Other")

    if int(acr) in range(1, 11):
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            # this needs to be changed to uuid once the api gets fixed
            cur.execute("SELECT asset_ip from tags where tag_key='" + c + "' and tag_value='" + v + "';")

            data = cur.fetchall()

            lumin_list = []

            for asset in data:
                # grab the first record, in this case the uuid
                uuid = asset[0]
                check_for_no = tag_checker(uuid, "NO", "UPDATE")
                if check_for_no == 'no':
                    check_match = tag_checker(uuid, c, v)
                    if check_match == 'yes':
                        lumin_list.append(uuid)
                else:
                    pass

            if not lumin_list:
                click.echo("We did not find a Tag with that Category or Value... "
                           "It's also possible the tag has not been applied to any assets\n")
                click.echo("If you think this is an error, surround your category and value in \"\"")
                exit()
            else:
                note = note + " - navi"
                # this needs to be changed to ID once the api is fixed
                lumin_payload = [{"acr_score": int(acr), "reason": choice, "note": note, "asset": [{"ipv4": lumin_list}]}]
                request_data('POST', '/api/v2/assets/bulk-jobs/acr', payload=lumin_payload)

    else:
        click.echo("You can't have a score below 1 or higher than 10")



import requests
import click
from sqlite3 import Error
from json import JSONDecodeError
from .database import new_db_connection
from tenable.io import TenableIO


def navi_version():
    return "navi-6.4.7"


def tenb_connection():
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT * from keys;")
            rows = cur.fetchall()
            for row in rows:
                access_key = row[0]
                secret_key = row[1]
            tio = TenableIO(access_key, secret_key, vendor='Casey Reid', product='navi', build=navi_version())
            return tio
    except:
        pass


def grab_headers():
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        cur = conn.cursor()
        try:
            cur.execute("SELECT * from keys;")
        except Error:
            click.echo("\nYou don't have any API keys!  Please enter your keys\n")
            exit()
        rows = cur.fetchall()
        for row in rows:
            access_key = row[0]
            secret_key = row[1]
    return {'Content-type': 'application/json', 'user-agent': navi_version(), 'X-ApiKeys': 'accessKey=' + access_key + ';secretKey=' + secret_key}


def request_no_response(method, url_mod, **kwargs):

    # set the Base URL
    url = "https://cloud.tenable.com"

    # check for params and set to None if not found
    try:
        params = kwargs['params']
    except KeyError:
        params = None

    # check for a payload and set to None if not found
    try:
        payload = kwargs['payload']
    except KeyError:
        payload = None

    try:
        r = requests.request(method, url + url_mod, headers=grab_headers(), params=params, json=payload, verify=True)
        if r.status_code == 200:
            click.echo("Success!\n")
        elif r.status_code == 404:
            click.echo('\nCheck your query...{}'.format(r))
        elif r.status_code == 429:
            click.echo("\nToo many requests at a time...\n {}".format(r))
        elif r.status_code == 400:
            click.echo("\nCheck your params.  Password complexity is the most common issue\n{}".format(r))
            click.echo()
        else:
            click.echo("Something went wrong...Don't be trying to hack me now {}".format(r))
    except ConnectionError:
        click.echo("Check your connection...You got a connection error")


def request_data(method, url_mod, **kwargs):

    # set the Base URL
    url = "https://cloud.tenable.com"

    # check for params and set to None if not found
    try:
        params = kwargs['params']
    except KeyError:
        params = None

    # check for a payload and set to None if not found
    try:
        payload = kwargs['payload']
    except KeyError:
        payload = None

    # Retry the download three times
    for x in range(1, 3):
        try:
            r = requests.request(method, url + url_mod, headers=grab_headers(), params=params, json=payload, verify=True)
            if r.status_code == 200:
                return r.json()

            if r.status_code == 202:
                # This response is for some successful posts.
                click.echo("\nSuccess!\n")
                break
            elif r.status_code == 404:
                click.echo('\nCheck your query...I can\'t find what you\'re looking for {}'.format(r))
                return r.json()
            elif r.status_code == 429:
                click.echo("\nToo many requests at a time...\n{}".format(r))
                break
            elif r.status_code == 400:
                click.echo("\nThe object you tried to create may already exist\n")
                click.echo("If you are changing scan ownership, there is a bug where 'empty' scans won't be moved")
                break
            elif r.status_code == 403:
                click.echo("\nYou are not authorized! You need to be an admin\n{}".format(r))
                break
            elif r.status_code == 409:
                click.echo("API Returned 409")
                break
            elif r.status_code == 504:
                click.echo("\nOne of the Threads and an issue during download...Retrying...\n{}".format(r))
                break
            else:
                click.echo("Something went wrong...Don't be trying to hack me now {}".format(r))
                break
        except ConnectionError:
            click.echo("Check your connection...You got a connection error. Retying")
            continue
        except JSONDecodeError:
            click.echo("Download Error or User enabled / Disabled ")
            continue



import csv
import click
from sqlite3 import Error
from .database import new_db_connection
from .api_wrapper import tenb_connection

tio = tenb_connection()


def tag_export(tag_list):
    click.echo("This export can take some time.  ~300 assets per minute.")
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:

        # Create our headers - We will Add these two our list in order
        header_list = ["IP Address", "Hostname", "FQDN", "UUID", "First Found", "Last Found", "Operating System",
                       "Mac Address", "Agent-UUID", "last Licensed Scan Date", 'Network', 'ACR', 'AES', 'AWS ID',
                       'Info', 'Low', 'Medium', 'High', 'Critical']
        cur = conn.cursor()
        try:
            cur.execute("SELECT * from assets;")
        except Error:
            print("\n No data! \n Please run 'navi update' first.\n")
            exit()
        data = cur.fetchall()

        # Crete a csv file object
        with open('bytag_lumin.csv', mode='w') as csv_file:
            agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')

            # write our Header information first
            agent_writer.writerow(header_list)

            # Loop through each asset
            for assets in data:
                export_list = []
                if assets[3] in tag_list:
                    for atr in assets:
                        # Cycle through the Database and populate the new list
                        export_list.append(atr)

                    asset_id = assets[3]  # Grab the UUID to make API calls

                    try:
                        asset_info = tio.workbenches.asset_info(asset_id)

                        for vuln in asset_info['counts']['vulnerabilities']['severities']:
                            export_list.append(vuln["count"])  # Add the vuln counts to the new list

                    except ConnectionError:
                        click.echo("Check your API keys or your internet connection")
                    # write to the CSV
                    agent_writer.writerow(export_list)
        click.echo("Export success! - bytag_lumin.csv")



from setuptools import setup, find_packages
from os import path


this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='navi pro',
    version='6.4.7',
    description="A command-line interface to Tenable.io",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Casey Reid",
    author_email="itprofguru@gmail.com",
    url="https://github.com/packetchaos/Navi",
    license="GNUv3",
    keywords='tenable tenable_io navi tio, lumin, navi pro, tio cli, tenable io cli, automation',
    packages=find_packages(exclude=['docs', 'tests']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'Click>=7.0',
        'requests',
        'flask',
        'IPy',
        'pytenable>=1.3',
        'arrow>=0.17.0',
        'boto3>=1.17.48'
    ],
    python_requires='>=3.0',
    extras_require={
    },
    entry_points={
        'console_scripts': [
            'navi=navi.cli:cli',
        ],
    },
)

